import * as i0 from '@angular/core';
import { InjectionToken, Optional, Inject, Directive, Input, Component, ChangeDetectionStrategy, ViewEncapsulation, ContentChildren, Injectable, inject, LOCALE_ID, Pipe, APP_INITIALIZER, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { formatDate, DOCUMENT, CommonModule } from '@angular/common';
import { __awaiter, __decorate, __param } from 'tslib';
import { startWith, map, distinctUntilChanged } from 'rxjs/operators';
import { Subscription, BehaviorSubject, of } from 'rxjs';
import * as i2 from '@bravobit/bb-foundation/utils';
import { UtilsModule } from '@bravobit/bb-foundation/utils';
import * as i2$1 from '@bravobit/bb-foundation';
import { WINDOW } from '@bravobit/bb-foundation';
import * as i1$1 from '@bravobit/bb-foundation/storage';

//
// The configuration that can be passed when
// the user initializes the localize module.
//
class LocalizeConfig {
}

//
// This is the injection token that will be injected
// into the service so that the all localize functions
// can retrieved and used.
//
const LOCALIZE_FUNCTION = new InjectionToken('localize_function');

const LOCALIZE_TRANSFORM = new InjectionToken('localize_transform');

//
// This function transform the value
// to a lowercase variant of the value.
//
class LocalizeLowercaseFunction {
    keyword() {
        return 'lowercase';
    }
    transform(value) {
        return (value !== null && value !== void 0 ? value : '').toLowerCase();
    }
}

//
// This function transform the value
// to a uppercase variant of the value.
//
class LocalizeUppercaseFunction {
    keyword() {
        return 'uppercase';
    }
    transform(value) {
        return (value !== null && value !== void 0 ? value : '').toUpperCase();
    }
}

//
// This function transform the value
// to a localized date.
//
class LocalizeDateFunction {
    keyword() {
        return 'date';
    }
    transform(value, locale) {
        const date = new Date(value);
        if (date.toString() === 'Invalid Date') {
            return '?';
        }
        return formatDate(date, 'longDate', locale);
    }
}

const extraEnglishDictionary = {
    'form-control-errors': {
        'unknown': 'The field has an unknown error.',
        'required': 'This field is required.',
        'pattern': 'Invalid format.',
        'email': 'The field does not match a valid email address.',
        'emailTaken': 'The e-mail is already taken.',
        'passwordMatch': 'The passwords don\'t match.',
        'confirm': 'The field does not match.',
        'fullNumber': 'The field should be a full number.',
        'min': 'Minimum amount should be {{min}}.',
        'max': 'Maximum amount should be {{max}}.',
        'invalidFileType': 'The file type is not allowed.',
        'maxFileSize': 'The file has exceed the maximum amount of {{maxSize}}.',
        'minDate': 'The first accepted date is {{date|date}}.',
        'maxDate': 'The last accepted date is {{date|date}}.',
        'invalidDate': 'The field contains an invalid date.',
        'minlength': 'Expected a minimum of {{requiredLength}} characters but got {{actualLength}} characters.',
        'maxlength': 'Expected a maximum of {{requiredLength}} characters but got {{actualLength}} characters.'
    },
    'tables': {
        'search': 'Search...',
        'loading': 'Loading...',
        'empty': 'There is nothing to find here.',
        'page': 'page',
        'error_datasource': 'No data source was provided.',
        'error_unauthorized': 'You need to be authenticated to view this data.',
        'error_forbidden': 'You do not have access to view this data.',
        'error_not_found': 'The data was not found.',
        'error_server': 'A server error occurred while fetching data from the server.',
        'error_down': 'It looks like the server is down.',
        'error_unknown': 'A unknown error occurred.'
    },
    'file-picker': {
        'choose': 'Choose file',
        'remove': 'Remove file',
        'drop': 'or drag and drop file here'
    },
    'times': {
        'tomorrow_at': 'Tomorrow at {{date}}',
        'today_at': 'Today at {{date}}',
        'yesterday_at': 'Yesterday at {{date}}'
    }
};

const extraDutchDictionary = {
    'form-control-errors': {
        'unknown': 'Het veld heeft een onbekende fout.',
        'required': 'Dit veld is verplicht.',
        'pattern': 'Ongeldig formaat.',
        'email': 'Het veld komt niet overeen met een geldig e-mailadres.',
        'emailTaken': 'Het e-mailadres is al bezet.',
        'passwordMatch': 'De wachtwoorden komen niet overeen.',
        'confirm': 'Het veld komt niet overeen.',
        'fullNumber': 'Het veld moet een volledig getal zijn.',
        'min': 'Minimum hoeveelheid moet {{min}} zijn.',
        'max': 'Maximum hoeveelheid moet {{max}} zijn.',
        'invalidFileType': 'Het type bestand is niet toegestaan.',
        'maxFileSize': 'Het bestand heeft de maximum hoeveelheid van {{maxSize}} overschreden.',
        'minDate': 'De eerst mogelijke datum is {{date|date}}.',
        'maxDate': 'De laatst mogelijke datum is {{date|date}}.',
        'invalidDate': 'Het veld bevat een ongeldige datum.',
        'minlength': 'Verwachtte minimaal {{requiredLength}} tekens, maar kreeg {{actualLength}} tekens.',
        'maxlength': 'Verwachtte maximaal {{requiredLength}} tekens, maar kreeg {{actualLength}} tekens.'
    },
    'tables': {
        'search': 'Zoeken...',
        'loading': 'Aan het laden...',
        'empty': 'Er is hier niets te vinden.',
        'page': 'pagina',
        'error_datasource': 'Er is geen gegevensbron verstrekt.',
        'error_unauthorized': 'U moet ingelogd zijn om deze gegevens te bekijken.',
        'error_forbidden': 'U hebt geen toegang om deze gegevens te bekijken.',
        'error_not_found': 'De gegevens zijn niet gevonden.',
        'error_server': 'Er is een serverfout opgetreden bij het ophalen van gegevens van de server.',
        'error_down': 'Het lijkt erop dat de server down is.',
        'error_unknown': 'Er is een onbekende fout opgetreden bij het ophalen van gegevens van de server.'
    },
    'file-picker': {
        'choose': 'Kies bestand',
        'remove': 'Verwijder bestand',
        'drop': 'of sleep het bestand hiernaartoe'
    },
    'times': {
        'tomorrow_at': 'Morgen om {{date}}',
        'today_at': 'Vandaag om {{date}}',
        'yesterday_at': 'Gisteren om {{date}}'
    }
};

class LocalizeMissingHandler {
    constructor() {
        this.log = (message) => { var _a; return (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, message); };
    }
    handle(params) {
        var _a;
        // Log to the console when the token was not found
        // if the user did not mark it as optional.
        if (!(params === null || params === void 0 ? void 0 : params.optional)) {
            this.log(`The token with name: "${params === null || params === void 0 ? void 0 : params.token}" was not found.`);
        }
        // Return the default token back.
        return (_a = params === null || params === void 0 ? void 0 : params.token) !== null && _a !== void 0 ? _a : null;
    }
}

class LocalizeDictionaryData {
    constructor(data) {
        this.data = data;
    }
}

class LocalizeDictionaryContext {
    constructor() {
        this.regexp = /^.*$/;
    }
    pattern(regexp) {
        this.regexp = regexp;
        return this;
    }
    matches(value) {
        const formatted = this.formatValue(value);
        const regex = (formatted !== null && formatted !== void 0 ? formatted : '')
            .replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&')
            .replace(/\\\*/g, '.*');
        return this.pattern(new RegExp(`^${regex}$`));
    }
    group(value) {
        const formatted = this.formatValue(value);
        return this.matches([formatted, '*']);
    }
    formatValue(value) {
        return Array.isArray(value)
            ? value.filter(item => !!item).join('.')
            : value;
    }
}

class LocalizeDictionary {
    constructor(_id, _locale, _supported, _dataValues, _localize) {
        this._id = _id;
        this._locale = _locale;
        this._supported = _supported;
        this._dataValues = _dataValues;
        this._localize = _localize;
        this._data = this.formatData(this._dataValues);
    }
    get id() {
        var _a;
        return (_a = this._id) !== null && _a !== void 0 ? _a : null;
    }
    get locale() {
        var _a;
        return (_a = this._locale) !== null && _a !== void 0 ? _a : null;
    }
    get supported() {
        var _a;
        return (_a = this._supported) !== null && _a !== void 0 ? _a : [];
    }
    get(token) {
        var _a, _b;
        const key = Array.isArray(token) ? token.join('.') : token;
        return (_b = (_a = this._data) === null || _a === void 0 ? void 0 : _a[key]) !== null && _b !== void 0 ? _b : null;
    }
    has(token) {
        return !!this.get(token);
    }
    all() {
        return this.some(context => context);
    }
    some(callback) {
        var _a;
        const defaultContext = new LocalizeDictionaryContext();
        const context = !!callback ? callback(defaultContext) : defaultContext;
        return Object.keys((_a = this._data) !== null && _a !== void 0 ? _a : {})
            .filter(key => context.regexp.test(key))
            .reduce((previous, current) => ([
            ...previous,
            { token: current, value: this.get(current) }
        ]), []);
    }
    compare(to) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if ((typeof ngDevMode === 'undefined' || ngDevMode)) {
                const dictionaryIds = (Array.isArray(to) ? to : [to]);
                const dictionaries = [];
                for (const dictionaryId of dictionaryIds) {
                    const dictionary = yield this._localize.load(dictionaryId);
                    dictionaries.push(dictionary);
                }
                const warnings = [];
                const items = this.all();
                for (const item of items) {
                    for (const dictionary of dictionaries) {
                        if (!dictionary.has(item === null || item === void 0 ? void 0 : item.token)) {
                            warnings.push({ dictionary: dictionary === null || dictionary === void 0 ? void 0 : dictionary.id, value: item === null || item === void 0 ? void 0 : item.token });
                        }
                    }
                }
                for (const dictionary of dictionaries) {
                    const items = dictionary.all();
                    for (const item of items) {
                        if (!this.has(item === null || item === void 0 ? void 0 : item.token)) {
                            warnings.push({ dictionary: this.id, value: item === null || item === void 0 ? void 0 : item.token });
                        }
                    }
                }
                const compareDictionaryIds = dictionaries.map(dictionary => dictionary === null || dictionary === void 0 ? void 0 : dictionary.id).join(', ');
                for (const warning of warnings) {
                    (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, `Dictionary "${warning.dictionary}" is missing token "${warning.value}"`);
                }
                (_b = console === null || console === void 0 ? void 0 : console.warn) === null || _b === void 0 ? void 0 : _b.call(console, `Total of ${warnings.length} warning(s) found while comparing "${this.id}" -> [${compareDictionaryIds}].`);
            }
        });
    }
    count() {
        var _a;
        return Object.keys((_a = this._data) !== null && _a !== void 0 ? _a : {}).length;
    }
    formatData(data, scope = null) {
        return Object.keys(data !== null && data !== void 0 ? data : {}).reduce((previous, current) => {
            var _a;
            const value = data === null || data === void 0 ? void 0 : data[current];
            if (typeof value === 'object' && value !== null) {
                const reformatted = this.formatData(value, current);
                return Object.assign(Object.assign({}, previous), reformatted);
            }
            const token = [scope, current].filter(item => !!item).join('.');
            if (previous[token] && (typeof ngDevMode === 'undefined' || ngDevMode)) {
                (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, `Duplicate key "${token}" was found, please verify your translation files.`);
            }
            return Object.assign(Object.assign({}, previous), { [token]: value });
        }, {});
    }
}

class LocalizeTransform {
    getMatches(value, substitutions) {
        // Validate our value exists, else return zero matches.
        if (value === undefined || value === null) {
            return [];
        }
        // Get the matching substitutions.
        const { start, end } = this.getSubstitutions(substitutions);
        // Compose a regex that will find the matches.
        const regex = new RegExp(`${start}([^${end}]+)${end}`, 'g');
        // Execute the regex to find matches.
        let match = regex.exec(value);
        const matches = [];
        while (match !== null) {
            matches.push({ match: match[0], token: match[1] });
            match = regex.exec(value);
        }
        // Return the processed value.
        return matches.filter(item => !!item);
    }
    getSubstitutions(value) {
        // Escape both substitutions so that they can be used.
        const escape = /[.*+?^${}()|[\]\\]/g;
        const start = value[0].replace(escape, '\\$&');
        const end = value[1].replace(escape, '\\$&');
        // Return the substitutions.
        return { start, end };
    }
}

let LocalizeInterpolateTransform = class LocalizeInterpolateTransform extends LocalizeTransform {
    constructor(config, functions) {
        var _a, _b, _c, _d;
        super();
        this.config = config;
        this.functions = functions;
        this.substitutions = (_c = (_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.substitutions) === null || _b === void 0 ? void 0 : _b.references) !== null && _c !== void 0 ? _c : ['{{', '}}'];
        this.functionsMap = ((_d = this.functions) !== null && _d !== void 0 ? _d : []).reduce((previous, current) => {
            previous[current === null || current === void 0 ? void 0 : current.keyword()] = current;
            return previous;
        }, {});
    }
    execute(value, data, localize) {
        const matches = this.getMatches(value, this.substitutions);
        // Replace all matches with the translated value.
        return matches.reduce((previous, current) => {
            // Try to parse the token.
            const interpolatedToken = this.parseInterpolationToken(current === null || current === void 0 ? void 0 : current.token, data, localize);
            // Replace the match with the translated token.
            return previous.replace(current === null || current === void 0 ? void 0 : current.match, interpolatedToken);
        }, value);
    }
    parseInterpolationToken(value, data, localize) {
        var _a;
        // Get the token and the methods by splitting
        // on the pipe symbol.
        const [token, ...methods] = (value !== null && value !== void 0 ? value : '').split('|').map(item => item.trim());
        // Get the value for the token from
        // the interpolation data.
        const interpolationValue = (_a = data === null || data === void 0 ? void 0 : data[token]) !== null && _a !== void 0 ? _a : 'null';
        // Loop through all the middleware methods and try to
        // conform the string to the given parameters.
        return methods.reduce((string, middlewareName) => {
            var _a, _b;
            // Try to retrieve the middleware from the collection.
            const middleware = (_a = this.functionsMap) === null || _a === void 0 ? void 0 : _a[middlewareName];
            // If the middleware is missing return an error
            // so that the user knows it was not found.
            if (!middleware) {
                return string;
            }
            // Execute the middleware function with the string.
            return middleware.transform(string, (_b = localize === null || localize === void 0 ? void 0 : localize.current) === null || _b === void 0 ? void 0 : _b.locale);
        }, `${interpolationValue}`);
    }
};
LocalizeInterpolateTransform = __decorate([
    __param(0, Optional()),
    __param(1, Optional()),
    __param(1, Inject(LOCALIZE_FUNCTION))
], LocalizeInterpolateTransform);

let LocalizeReferenceTransform = class LocalizeReferenceTransform extends LocalizeTransform {
    constructor(config) {
        var _a, _b, _c;
        super();
        this.config = config;
        this.substitutions = (_c = (_b = (_a = this.config) === null || _a === void 0 ? void 0 : _a.substitutions) === null || _b === void 0 ? void 0 : _b.references) !== null && _c !== void 0 ? _c : ['#(', ')'];
    }
    execute(value, _, localize) {
        const matches = this.getMatches(value, this.substitutions);
        // Replace all matches with the translated value.
        return matches.reduce((previous, current) => {
            // Try to translate the token.
            const translatedToken = localize.translate(current === null || current === void 0 ? void 0 : current.token);
            // Replace the match with the translated token.
            return previous.replace(current === null || current === void 0 ? void 0 : current.match, translatedToken);
        }, value);
    }
};
LocalizeReferenceTransform = __decorate([
    __param(0, Optional())
], LocalizeReferenceTransform);

class BbLocalizeTemplate {
    constructor(templateRef) {
        this.templateRef = templateRef;
        // Inputs.
        this.name = null;
    }
}
BbLocalizeTemplate.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalizeTemplate, deps: [{ token: i0.TemplateRef }], target: i0.ɵɵFactoryTarget.Directive });
BbLocalizeTemplate.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "15.0.3", type: BbLocalizeTemplate, selector: "[bbLocalizeTemplate]", inputs: { name: ["bbLocalizeTemplate", "name"] }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalizeTemplate, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bbLocalizeTemplate]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }]; }, propDecorators: { name: [{
                type: Input,
                args: ['bbLocalizeTemplate']
            }] } });

class BbLocalizeString {
    constructor(_changeDetection) {
        this._changeDetection = _changeDetection;
        // Inputs.
        this.substitutions = ['[', ']'];
        // Data.
        this.components = [];
        // Subscriptions.
        this._subscription = new Subscription();
        // State.
        this._value = null;
    }
    set value(value) {
        this._value = value;
        this.revalidate();
    }
    get value() {
        return this._value;
    }
    ngAfterViewInit() {
        this.observeItemChanges();
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    observeItemChanges() {
        const subscription = this.items.changes.pipe(startWith(0)).subscribe(() => this.revalidate());
        this._subscription.add(subscription);
    }
    revalidate() {
        var _a, _b;
        const items = (_b = (_a = this.items) === null || _a === void 0 ? void 0 : _a.toArray()) !== null && _b !== void 0 ? _b : [];
        if (items.length <= 0) {
            return;
        }
        this.setComponents(items);
    }
    setComponents(items) {
        const [start, end] = this.substitutions;
        const templates = items.reduce((previous, current) => {
            var _a, _b;
            const template = (_a = current === null || current === void 0 ? void 0 : current.templateRef) !== null && _a !== void 0 ? _a : null;
            const name = (_b = current === null || current === void 0 ? void 0 : current.name) !== null && _b !== void 0 ? _b : null;
            if (!template || !name) {
                return previous;
            }
            return Object.assign(Object.assign({}, previous), { [`${start}${name}${end}`]: template });
        }, {});
        this.components = this.getStringComponents(this._value)
            .map(component => { var _a, _b; return (_b = (_a = templates === null || templates === void 0 ? void 0 : templates[component]) !== null && _a !== void 0 ? _a : component) !== null && _b !== void 0 ? _b : null; })
            .filter(item => !!item);
        this._changeDetection.detectChanges();
    }
    getStringComponents(value) {
        const [start, end] = this.getSubstitutions();
        const regex = new RegExp(`(${start}[a-zA-Z]+${end})`, 'g');
        return value.split(regex);
    }
    getSubstitutions() {
        const [start, end] = this.substitutions;
        const escape = /[.*+?^${}()|[\]\\]/g;
        return [
            start.replace(escape, '\\$&'),
            end.replace(escape, '\\$&')
        ];
    }
}
BbLocalizeString.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalizeString, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component });
BbLocalizeString.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "15.0.3", type: BbLocalizeString, selector: "bb-localize-string, [bb-localize-string]", inputs: { substitutions: "substitutions", value: "value" }, host: { classAttribute: "bb-localize-string" }, queries: [{ propertyName: "items", predicate: BbLocalizeTemplate }], ngImport: i0, template: "<ng-container *ngFor=\"let component of components\">\n    <ng-template [bbTemplate]=\"component\">{{ component }}</ng-template>\n</ng-container>\n", styles: [".bb-localize-string{display:inline}\n"], dependencies: [{ kind: "directive", type: i1.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2.BbTemplate, selector: "[bbTemplate]", inputs: ["bbTemplate"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush, encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalizeString, decorators: [{
            type: Component,
            args: [{ selector: 'bb-localize-string, [bb-localize-string]', changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, preserveWhitespaces: false, host: {
                        'class': 'bb-localize-string'
                    }, template: "<ng-container *ngFor=\"let component of components\">\n    <ng-template [bbTemplate]=\"component\">{{ component }}</ng-template>\n</ng-container>\n", styles: [".bb-localize-string{display:inline}\n"] }]
        }], ctorParameters: function () { return [{ type: i0.ChangeDetectorRef }]; }, propDecorators: { items: [{
                type: ContentChildren,
                args: [BbLocalizeTemplate]
            }], substitutions: [{
                type: Input
            }], value: [{
                type: Input
            }] } });

class Localize {
    constructor(_storage, _languages, _missingHandler, _config, _window, _document, _transforms) {
        var _a, _b, _c, _d;
        this._storage = _storage;
        this._languages = _languages;
        this._missingHandler = _missingHandler;
        this._config = _config;
        this._window = _window;
        this._document = _document;
        this._transforms = _transforms;
        // Readonly data.
        this._storageToken = (_b = (_a = this._config) === null || _a === void 0 ? void 0 : _a.storageToken) !== null && _b !== void 0 ? _b : 'bb-language-dictionary-id';
        this._availableLanguages = (_d = (_c = this === null || this === void 0 ? void 0 : this._config) === null || _c === void 0 ? void 0 : _c.languages) !== null && _d !== void 0 ? _d : [];
        this._bestMatchingLanguage = this.retrieveBestMatchingLanguage();
        this._loadedDictionaries = new Map();
        // Current dictionary.
        this._current$ = new BehaviorSubject(null);
        // State changes.
        this.dictionaryChanges$ = this._current$.asObservable();
        this.convertToParams = (tokenOrParams) => {
            if (typeof tokenOrParams !== 'string') {
                // Destruct the params from the object.
                const { token, optional, dictionary, data } = tokenOrParams;
                // Make sure all parameters are valid
                // by type safe adding them.
                return { token: token, optional: optional !== null && optional !== void 0 ? optional : false, dictionary: dictionary !== null && dictionary !== void 0 ? dictionary : null, data: data !== null && data !== void 0 ? data : {} };
            }
            // If the parameter is a string fill in the blanks.
            return { token: tokenOrParams, optional: false, dictionary: null, data: {} };
        };
        this.renderDocumentLanguage();
    }
    get languages() {
        var _a;
        return (_a = this._availableLanguages) !== null && _a !== void 0 ? _a : [];
    }
    get current() {
        return this._current$.getValue();
    }
    load(id) {
        var _a, _b, _c, _d, _e, _f;
        return __awaiter(this, void 0, void 0, function* () {
            if (id === null || id === undefined) {
                throw new Error(`Cannot load language with id: "${id}".`);
            }
            if (this._loadedDictionaries.has(id)) {
                return this._loadedDictionaries.get(id);
            }
            const language = (_a = this.languages) === null || _a === void 0 ? void 0 : _a.find(language => (language === null || language === void 0 ? void 0 : language.id) === id);
            if (!language || !(language === null || language === void 0 ? void 0 : language.data)) {
                (_b = console === null || console === void 0 ? void 0 : console.warn) === null || _b === void 0 ? void 0 : _b.call(console, `No language was found with id: "${id}"; Defaulting to first available language.`);
                const languageId = (_e = (_d = (_c = this.languages) === null || _c === void 0 ? void 0 : _c[0]) === null || _d === void 0 ? void 0 : _d.id) !== null && _e !== void 0 ? _e : null;
                return this.load(languageId);
            }
            const dictionaryDataValues = yield ((_f = language === null || language === void 0 ? void 0 : language.data) === null || _f === void 0 ? void 0 : _f.call(language));
            const dictionary = this.createDictionary(dictionaryDataValues, language);
            this._loadedDictionaries.set(id, dictionary);
            return dictionary;
        });
    }
    loadAllLanguages() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            for (const language of this.languages) {
                yield this.load((_a = this.current) === null || _a === void 0 ? void 0 : _a.id);
            }
        });
    }
    switch(id, options) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            const dictionary = yield this.load(id);
            this._current$.next(dictionary);
            const cache = (_a = options === null || options === void 0 ? void 0 : options.cache) !== null && _a !== void 0 ? _a : true;
            if (cache) {
                this._storage.cookie.set(this._storageToken, dictionary === null || dictionary === void 0 ? void 0 : dictionary.id, {});
            }
            const reload = (_b = options === null || options === void 0 ? void 0 : options.reload) !== null && _b !== void 0 ? _b : false;
            if (reload) {
                (_e = (_d = (_c = this._window) === null || _c === void 0 ? void 0 : _c.location) === null || _d === void 0 ? void 0 : _d.reload) === null || _e === void 0 ? void 0 : _e.call(_d);
            }
            return dictionary;
        });
    }
    select(id) {
        if (!this._loadedDictionaries.has(id)) {
            throw new Error(`Dictionary with id: "${id}" not loaded; Load dictionary first before using .select(:id).`);
        }
        return this._loadedDictionaries.get(id);
    }
    initialize() {
        return () => __awaiter(this, void 0, void 0, function* () {
            var _a;
            yield this.switch((_a = this._bestMatchingLanguage) === null || _a === void 0 ? void 0 : _a.id, { reload: false, cache: false });
            return this.compareDictionaries();
        });
    }
    clear() {
        this._storage.cookie.remove(this._storageToken);
    }
    transform(value, data = {}) {
        var _a;
        return ((_a = this._transforms) !== null && _a !== void 0 ? _a : []).reduce((previous, current) => {
            return current.execute(previous, data, this);
        }, value);
    }
    translate(tokenOrParams) {
        // 1. Convert to the params.
        const params = this.convertToParams(tokenOrParams);
        // 2. Try to find the translation in the dictionary.
        const dictionary = this.getDictionary(params === null || params === void 0 ? void 0 : params.dictionary);
        const searchResult = !!dictionary ? dictionary.get(params === null || params === void 0 ? void 0 : params.token) : null;
        // 3. Verify the translation was found.
        if (!searchResult) {
            return this._missingHandler.handle(params);
        }
        return this.transform(searchResult, params === null || params === void 0 ? void 0 : params.data);
    }
    retrieveBestMatchingLanguage() {
        var _a, _b, _c;
        return (_b = (_a = this.getLanguageFromStorage()) !== null && _a !== void 0 ? _a : this.getBestBrowserMatchLanguage()) !== null && _b !== void 0 ? _b : (_c = this._availableLanguages) === null || _c === void 0 ? void 0 : _c[0];
    }
    getLanguageFromStorage() {
        // Get id from the storage.
        const id = this._storage.cookie.get(this._storageToken);
        // Return the language based on the storage id.
        return this._availableLanguages.find(item => (item === null || item === void 0 ? void 0 : item.id) === id);
    }
    getBestBrowserMatchLanguage() {
        // Loop through all browser languages and return the
        // best matching language.
        return this._languages.all.map(({ code, region }) => {
            const languageCode = [code, region].filter(data => !!data).join('-');
            return this._availableLanguages.find(item => { var _a; return (_a = item === null || item === void 0 ? void 0 : item.supported) === null || _a === void 0 ? void 0 : _a.includes(languageCode); });
        }).filter(item => !!item).find((_, index) => index === 0);
    }
    createDictionary(data, language) {
        return new LocalizeDictionary(language === null || language === void 0 ? void 0 : language.id, language === null || language === void 0 ? void 0 : language.locale, language === null || language === void 0 ? void 0 : language.supported, data === null || data === void 0 ? void 0 : data.data, this);
    }
    getDictionary(id) {
        if (id === null || id === undefined) {
            return this.current;
        }
        return this.select(id);
    }
    renderDocumentLanguage() {
        var _a, _b, _c, _d, _e;
        const element = (_a = this._document) === null || _a === void 0 ? void 0 : _a.documentElement;
        if (!element) {
            return;
        }
        element.lang = (_d = (_c = (_b = this._bestMatchingLanguage) === null || _b === void 0 ? void 0 : _b.supported) === null || _c === void 0 ? void 0 : _c[0]) !== null && _d !== void 0 ? _d : (_e = this._bestMatchingLanguage) === null || _e === void 0 ? void 0 : _e.locale;
    }
    compareDictionaries() {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const compareDictionaries = (_a = this._config.compareDictionaries) !== null && _a !== void 0 ? _a : false;
            if (!compareDictionaries || !this.current) {
                return;
            }
            const languageIds = (_d = (_c = ((_b = this.languages) !== null && _b !== void 0 ? _b : [])) === null || _c === void 0 ? void 0 : _c.filter(language => { var _a; return (language === null || language === void 0 ? void 0 : language.id) !== ((_a = this.current) === null || _a === void 0 ? void 0 : _a.id); })) === null || _d === void 0 ? void 0 : _d.map(language => language === null || language === void 0 ? void 0 : language.id);
            return yield this.current.compare(languageIds);
        });
    }
}
Localize.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: Localize, deps: [{ token: i1$1.Storage }, { token: i2$1.Languages }, { token: LocalizeMissingHandler }, { token: LocalizeConfig, optional: true }, { token: WINDOW, optional: true }, { token: DOCUMENT, optional: true }, { token: LOCALIZE_TRANSFORM, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
Localize.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: Localize });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: Localize, decorators: [{
            type: Injectable
        }], ctorParameters: function () {
        return [{ type: i1$1.Storage }, { type: i2$1.Languages }, { type: LocalizeMissingHandler }, { type: LocalizeConfig, decorators: [{
                        type: Optional
                    }] }, { type: Window, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [WINDOW]
                    }] }, { type: Document, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [DOCUMENT]
                    }] }, { type: undefined, decorators: [{
                        type: Optional
                    }, {
                        type: Inject,
                        args: [LOCALIZE_TRANSFORM]
                    }] }];
    } });

const LOCALIZE_ID = new InjectionToken('An abstraction over locale changes', {
    factory: () => {
        var _a, _b;
        const localize = inject(Localize, { optional: true });
        if (!localize) {
            return of(inject(LOCALE_ID));
        }
        return localize.dictionaryChanges$.pipe(map(dictionary => { var _a; return (_a = dictionary === null || dictionary === void 0 ? void 0 : dictionary.locale) !== null && _a !== void 0 ? _a : null; }), distinctUntilChanged(), startWith((_b = (_a = localize === null || localize === void 0 ? void 0 : localize.current) === null || _a === void 0 ? void 0 : _a.locale) !== null && _b !== void 0 ? _b : null));
    }
});

class BbLocalize {
    constructor(_localize) {
        this._localize = _localize;
    }
    transform(tokenOrParams, args) {
        var _a;
        // Combine the params with the options to form the localize params.
        const params = this.combineParamsWithOptions(tokenOrParams, args);
        // Validate that the localize service exists in the pipe.
        if (!this._localize) {
            return (_a = params === null || params === void 0 ? void 0 : params.token) !== null && _a !== void 0 ? _a : null;
        }
        // Try to localize the value.
        return this._localize.translate(params);
    }
    combineParamsWithOptions(tokenOrParams, args) {
        // Gather the extras.
        const extras = args !== null && args !== void 0 ? args : {};
        // Loop through all the keys to find the set options.
        const options = Object.keys(extras).reduce((previous, current) => {
            if (typeof (extras === null || extras === void 0 ? void 0 : extras[current]) === 'undefined') {
                return previous;
            }
            return Object.assign(Object.assign({}, previous), { [current]: extras === null || extras === void 0 ? void 0 : extras[current] });
        }, {});
        // Convert the string token to a params object.
        const params = typeof tokenOrParams === 'string'
            ? { token: tokenOrParams, optional: false, data: {}, dictionary: null }
            : tokenOrParams;
        // Return the combined params and options.
        return Object.assign(Object.assign({}, params), options);
    }
}
BbLocalize.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalize, deps: [{ token: Localize, optional: true }], target: i0.ɵɵFactoryTarget.Pipe });
BbLocalize.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "15.0.3", ngImport: i0, type: BbLocalize, name: "bbLocalize" });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: BbLocalize, decorators: [{
            type: Pipe,
            args: [{
                    name: 'bbLocalize'
                }]
        }], ctorParameters: function () {
        return [{ type: Localize, decorators: [{
                        type: Optional
                    }] }];
    } });

class LocalizeModule {
    static forRoot(config) {
        var _a, _b, _c;
        if (((_a = config === null || config === void 0 ? void 0 : config.languages) === null || _a === void 0 ? void 0 : _a.length) <= 0) {
            throw new Error('One or more languages should be supplied.');
        }
        if ((config === null || config === void 0 ? void 0 : config.compareDictionaries) && (config === null || config === void 0 ? void 0 : config.mode) === 'manual') {
            (_b = console === null || console === void 0 ? void 0 : console.warn) === null || _b === void 0 ? void 0 : _b.call(console, 'You cannot enable "compareDictionaries" when having mode set to "manual"; This setting will be ignored.');
        }
        const mode = (_c = config === null || config === void 0 ? void 0 : config.mode) !== null && _c !== void 0 ? _c : 'automatic';
        return {
            ngModule: LocalizeModule,
            providers: [
                // Angular providers.
                ...(mode === 'automatic' ? [
                    { provide: APP_INITIALIZER, deps: [Localize], useFactory: initializeLocale, multi: true },
                    { provide: LOCALE_ID, deps: [Localize], useFactory: getLocale }
                ] : []),
                // Config provider.
                { provide: LocalizeConfig, useValue: config },
                // Function providers.
                { provide: LOCALIZE_FUNCTION, useClass: LocalizeLowercaseFunction, multi: true },
                { provide: LOCALIZE_FUNCTION, useClass: LocalizeUppercaseFunction, multi: true },
                { provide: LOCALIZE_FUNCTION, useClass: LocalizeDateFunction, multi: true },
                // Transform providers.
                { provide: LOCALIZE_TRANSFORM, useClass: LocalizeReferenceTransform, deps: [LocalizeConfig], multi: true },
                { provide: LOCALIZE_TRANSFORM, useClass: LocalizeInterpolateTransform, deps: [LocalizeConfig, LOCALIZE_FUNCTION], multi: true },
                // Handler providers.
                { provide: LocalizeMissingHandler, useClass: LocalizeMissingHandler },
                // Service providers.
                Localize
            ]
        };
    }
}
LocalizeModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: LocalizeModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
LocalizeModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "15.0.3", ngImport: i0, type: LocalizeModule, declarations: [BbLocalize,
        BbLocalizeString,
        BbLocalizeTemplate], imports: [CommonModule, UtilsModule], exports: [BbLocalize,
        BbLocalizeString,
        BbLocalizeTemplate] });
LocalizeModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: LocalizeModule, imports: [CommonModule, UtilsModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "15.0.3", ngImport: i0, type: LocalizeModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, UtilsModule],
                    declarations: [
                        BbLocalize,
                        BbLocalizeString,
                        BbLocalizeTemplate
                    ],
                    exports: [
                        BbLocalize,
                        BbLocalizeString,
                        BbLocalizeTemplate
                    ]
                }]
        }] });
function initializeLocale(localize) {
    return localize.initialize();
}
function getLocale(localize) {
    var _a, _b;
    return (_b = (_a = localize === null || localize === void 0 ? void 0 : localize.current) === null || _a === void 0 ? void 0 : _a.locale) !== null && _b !== void 0 ? _b : null;
}

/**
 * Generated bundle index. Do not edit.
 */

export { BbLocalize, BbLocalizeString, BbLocalizeTemplate, LOCALIZE_FUNCTION, LOCALIZE_ID, LOCALIZE_TRANSFORM, Localize, LocalizeConfig, LocalizeDateFunction, LocalizeDictionary, LocalizeDictionaryContext, LocalizeDictionaryData, LocalizeInterpolateTransform, LocalizeLowercaseFunction, LocalizeMissingHandler, LocalizeModule, LocalizeReferenceTransform, LocalizeTransform, LocalizeUppercaseFunction, extraDutchDictionary, extraEnglishDictionary, getLocale, initializeLocale };
//# sourceMappingURL=bravobit-bb-foundation-localize.mjs.map
